(function() {

  'use strict';

  angular
    .module('app')
    .controller('DenunciaController', DenunciaController);

  DenunciaController.$inject = ['$cordovaDialogs', '$scope', '$state', '$http', '$cordovaCamera', '$ionicLoading', '$cordovaFileTransfer', 'GlobalService', 'ToastService', '$cordovaGeolocation'];

  function DenunciaController($cordovaDialogs, $scope, $state, $http, $cordovaCamera, $ionicLoading, $cordovaFileTransfer, GlobalService, ToastService, $cordovaGeolocation) {
    var vm = this;

    vm.takePhoto = takePhoto;
    vm.getPhoto = getPhoto;
    vm.successPhotoCapture = successPhotoCapture;
    vm.sent = sent;
    vm.isValid = isValid;

    activate();

    $scope.$on('$ionicView.beforeEnter', function() {
      $cordovaGeolocation
        .getCurrentPosition({timeout: 10000, enableHighAccuracy: false})
        .then(function (position) {
          vm.latitude = position.coords.latitude;
          vm.longitude = position.coords.longitude;
        }, function(err) {});
    });

    function activate() {
      vm.denuncia = { anonima: false };
    }

    function getPhoto() {
      vm.takePhoto(Camera.PictureSourceType.PHOTOLIBRARY);
    }

    function takePhoto(type) {
      type = (angular.isDefined(type)) ? type : Camera.PictureSourceType.CAMERA;

      if(ionic.Platform.isWebView()) {
        $cordovaCamera.getPicture({
          quality: 100,
          targetWidth: 800,
          targetHeight: 600,
          saveToPhotoAlbum: false,
          correctOrientation: true,
          sourceType: type,
          encodingType: Camera.EncodingType.JPEG,
          destinationType: Camera.DestinationType.DATA_URL
        }).then(function(imageData) {
          vm.successPhotoCapture(imageData);
        }, function(error) {
          ToastService.show("Não foi possível obter a foto.");
          console.log(error);
        });
      } else {
        vm.denuncia.imageURI = GlobalService.defaultImageData;
      }
    }

    function successPhotoCapture(imageData) {
      vm.denuncia.imageURI = GlobalService.base64Prefix + imageData;
    }

    function sent(form) {
      if(isValid(form)) {
        $ionicLoading.show({ template: 'Enviando...' });

        var denuncia = angular.copy(vm.denuncia);

        denuncia.dispositivoId = GlobalService.uuid;
        denuncia.latitude = vm.latitude;
        denuncia.longitude = vm.longitude;

        if(vm.denuncia.imageURI) {
          denuncia.imageBase64 = vm.denuncia.imageURI.replace(GlobalService.base64Prefix, "");
          denuncia.imageURI = null;
        }

        denuncia.token = md5('procon' + denuncia.dispositivoId + 'mobile');

        $http.post(GlobalService.serverBaseUri + "/denuncias/salvar", denuncia, {timeout: 75000} )
          .then(function(result) {
            $ionicLoading.hide();
            $cordovaDialogs.alert('Denúncia enviada com sucesso.', '', 'Ok');
            activate();
          }, function(error) {
            $ionicLoading.hide();
            $cordovaDialogs.alert('Não foi possível enviar a denúncia. Tente novamente mais tarde.', 'Atenção', 'Ok');
          });
        }
    }

    function isValid(form) {
      var valid = true;

      if(!form.$valid) {
          valid = false;
          ToastService.show("Favor verificar os campos obrigatórios");
        }

      if(valid && vm.denuncia.cnpjDenunciado && !CNPJ.isValid(vm.denuncia.cnpjDenunciado)) {
        valid = false;
        ToastService.show("O CNPJ deve ser válido.");
      }

      return valid;
    }
  }

})();
